/*
 * Some usefull macros for Linux/MIPS assembler code
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1995 by Ralf Baechle
 *
 */
#ifndef	__ASM_ASM_H
#define	__ASM_ASM_H

#include <asm/regdef.h>

#ifndef CAT
#ifdef __STDC__
#define __CAT(str1,str2) str1##str2
#else
#define __CAT(str1,str2) str1/**/str2
#endif
#define CAT(str1,str2) __CAT(str1,str2)
#endif

/*
 * For now...
 */
#define C_LABEL_PREFIX	_

/*
 * C_LABEL - build label for function
 */
#define C_LABEL(name) CAT(C_LABEL_PREFIX,name)

/*
 * PTR - pointer
 */
#if __mips == 3
#define PTR	.quad
#define PTRSIZE	8
#define PTRLOG	3
#else
#define PTR	.word
#define PTRSIZE	4
#define PTRLOG	2
#endif

/*
 * LEAF - declare leaf routine
 */
#define	LEAF(symbol)                                     \
			.globl	C_LABEL(symbol);         \
			.align	2;                       \
			.ent	C_LABEL(symbol),0;       \
C_LABEL(symbol):	.frame	sp,0,ra

/*
 * NESTED - declare nested routine entry point
 */
#define	NESTED(symbol, framesize, rpc)                  \
			.globl	C_LABEL(symbol);        \
			.align	2;                      \
			.ent	C_LABEL(symbol),0;      \
C_LABEL(symbol):	.frame	sp, framesize, rpc

/*
 * END - mark end of function
 */
#define	END(function)                                   \
			.end	C_LABEL(function)

/*
 * EXPORT - export definition of symbol
 */
#define	EXPORT(symbol)                                  \
			.globl	C_LABEL(symbol);        \
C_LABEL(symbol):

/*
 * ABS - export absolute symbol
 */
#define	ABS(symbol,value)                               \
			.globl	C_LABEL(symbol);        \
C_LABEL(symbol)		=	value

#define	PANIC(msg)                                      \
			la	a0,8f;                  \
			jal	C_LABEL(panic);         \
			nop;                            \
9:			b	9b;                     \
			TEXT(msg)

/*
 * Print formated string
 */
#define PRINT(string)                                   \
			la	a0,8f;                  \
			jal	C_LABEL(printk);        \
			nop;                            \
			TEXT(string)

#define	TEXT(msg)                                       \
			.data;                          \
8:			.asciiz	msg;                    \
			.text

/*
 * Move to kernel mode and disable interrupts
 */
#define CLI                                              \
		mfc0	k0,CP0_STATUS;                   \
		ori	k0,k0,0x1f;                      \
		xori	k0,k0,0x1f;                      \
		mtc0	k0,CP0_STATUS

/*
 * Move to kernel mode and enable interrupts
 */
#define STI                                             \
		mfc0	k0,CP0_STATUS;                  \
		ori	k0,k0,0x1f;                     \
		xori	k0,k0,0x1e;                     \
		mtc0	k0,CP0_STATUS

/*
 * Special nop to fill load delay slots
 */
#ifndef __R4000__
#define NOP     nop
#else
#define NOP
#endif

/*
 * R8000/R10000 (MIPS ISA IV) pref instruction.
 * Use with .set noreorder only!
 */
#if defined (CONFIG_R8000) || defined(CONFIG_R10000)
#define PREF(hint,addr)                                 \
		pref	hint,addr
#define PREFX(hint,addr)                                \
		prefx	hint,addr
#else
#define PREF
#define PREFX
#endif

/*
 * R8000/R10000 (MIPS ISA IV) movn/movz instructions and
 * equivalents for old CPUs. Use with .set noreorder only!
 */
#if defined (CONFIG_R8000) || defined (CONFIG_R10000)
#define MOVN(rd,rs,rt)                                  \
		movn	rd,rs,rt
#define MOVZ(rd,rs,rt)                                  \
		movz	rd,rs,rt
#elif defined (CONFIG_R4000) || defined (CONFIG_R6000)
#define MOVN(rd,rs,rt)                                  \
		bnezl	rt,9f                           \
		move	rd,rs                           \
9:
#define MOVZ(rd,rs,rt)                                  \
		beqzl	rt,9f                           \
		movz	rd,rt                           \
9:
#else /* R2000, R3000 */
#define MOVN(rd,rs,rt)                                  \
		beqz	rt,9f                           \
		nop                                     \
		move	rd,rs                           \
9:
#define MOVZ(rd,rs,rt)                                  \
		bneqz	rt,9f                           \
		nop                                     \
		movz	rd,rt                           \
9:
#endif

#endif /* __ASM_ASM_H */
